use crate::{error::AdapterError, services::ObjectStorageClient};
use async_trait::async_trait;
use aws_config::{BehaviorVersion, Region};
use aws_sdk_s3::{Client, Config, config::Credentials};
use std::env;

#[derive(Clone, Debug)]
pub struct S3ClientConfig {
    pub access_key: String,
    secret_key: String,
    endpoint: String,
    pub bucket: String,
    region: String,
}

#[derive(Clone)]
pub struct S3Client {
    client: Client,
    pub client_config: S3ClientConfig,
}

impl S3ClientConfig {
    pub fn new(
        access_key: &str,
        secret_key: &str,
        endpoint: &str,
        bucket: &str,
        region: &str,
    ) -> Result<Self, Box<dyn std::error::Error>> {
        Ok(S3ClientConfig {
            access_key: access_key.to_owned(),
            secret_key: secret_key.to_owned(),
            endpoint: endpoint.to_owned(),
            bucket: bucket.to_owned(),
            region: region.to_owned(),
        })
    }

    pub fn from_env() -> Result<Self, Box<dyn std::error::Error>> {
        Ok(S3ClientConfig {
            access_key: env::var("S3_ACCESS_KEY")
                .map_err(|_| "S3_ACCESS_KEY environment variable not set")?,
            secret_key: env::var("S3_SECRET_KEY")
                .map_err(|_| "S3_SECRET_KEY environment variable not set")?,
            endpoint: env::var("S3_ENDPOINT")
                .unwrap_or_else(|_| "us-ord-1.linodeobjects.com".to_string()),
            bucket: env::var("S3_BUCKET").map_err(|_| "S3_BUCKET environment variable not set")?,
            region: env::var("S3_REGION").unwrap_or_else(|_| "us-ord".to_string()),
        })
    }
}

impl S3Client {
    pub fn new(config: &S3ClientConfig) -> Self {
        let credentials = Credentials::new(
            &config.access_key,
            &config.secret_key,
            None,
            None,
            "linode-object-storage",
        );

        let s3_config = Config::builder()
            .behavior_version(BehaviorVersion::latest())
            .region(Region::new(config.region.clone()))
            .endpoint_url(format!("https://{}", config.endpoint))
            .credentials_provider(credentials)
            .force_path_style(false)
            .build();

        Self {
            client: Client::from_conf(s3_config),
            client_config: config.clone(),
        }
    }
}

#[async_trait]
impl ObjectStorageClient for S3Client {
    type Error = AdapterError;

    async fn put_object(&self, bucket: &str, key: &str, data: Vec<u8>) -> Result<(), Self::Error> {
        println!("Uploading to S3 (or S3 like) Object Storage...");
        println!("Bucket: {}", bucket);

        let _ = self
            .client
            .put_object()
            .bucket(bucket)
            .key(key)
            .body(data.into())
            .acl(aws_sdk_s3::types::ObjectCannedAcl::PublicRead)
            .content_type("application/xml")
            .send()
            .await
            .unwrap();

        Ok(())
    }

    async fn get_object(&self, _bucket: &str, _key: &str) -> Result<Vec<u8>, Self::Error> {
        todo!("not impl")
    }

    async fn delete_object(&self, _bucket: &str, _key: &str) -> Result<(), Self::Error> {
        todo!("not impl")
    }

    async fn list_objects(
        &self,
        _bucket: &str,
        _prefix: Option<&str>,
    ) -> Result<Vec<String>, Self::Error> {
        todo!("not impl")
    }

    async fn object_exists(&self, _bucket: &str, _key: &str) -> Result<bool, Self::Error> {
        todo!("not impl")
    }
}

impl Default for S3ClientConfig {
    fn default() -> Self {
        S3ClientConfig::from_env().unwrap()
    }
}
