use crate::utils::{
    request::{Request, Response},
    task_log,
};
use cache::KeysInterface;
use storage::services::{aws::S3Client, ObjectStorageClient};

pub fn register(pool: &sqlx::Pool<sqlx::Postgres>, cache: &cache::Pool, s3_client: &S3Client) {
    let p = pool.clone();
    let c = cache.clone();
    let s3 = s3_client.to_owned();

    tokio::spawn(async move {
        let _ = upload_sitemap(&p, &c, s3).await;
    });
}

async fn upload_sitemap(
    pool: &sqlx::Pool<sqlx::Postgres>,
    cache: &cache::Pool,
    s3_client: S3Client,
) -> Result<(), Box<dyn std::error::Error>> {
    // start task logging
    task_log::start(3, pool).await?;

    // get request and request the things
    let request = Request::new();
    let sitemap_url = format!("{}/posts/sitemap", request.base_url);
    let sitemap_result = request.request_url::<String>(&sitemap_url).await.unwrap();

    // upload the sucker to obj storage
    if let Response::Xml(sitemap) = sitemap_result {
        let cached: &Option<String> = &cache.get(String::from("sitemap")).await.unwrap_or(None);
        let cache_clone = cache.clone();
        if let Some(cached_value) = cached {
            if *cached_value == sitemap {
                println!("Response is the same in the cache, exiting");
                return Ok(());
            }
        }
        let s = sitemap.clone();

        let _ = s3_client
            .put_object(
                s3_client.client_config.bucket.as_str(),
                "sitemap.xml",
                sitemap.as_bytes().to_vec(),
            )
            .await?;

        tokio::spawn(async move {
            cache_clone
                .set::<String, String, &String>(
                    String::from("sitemap"),
                    &s,
                    Some(cache::Expiration::EX(3600)),
                    None,
                    false,
                )
                .await
                .unwrap();
        });
        println!("Finished uploading sitemap!");
    }

    Ok(())
}
